import * as d3 from 'd3';
// https://observablehq.com/@d3/d3-bin
// https://www.d3-graph-gallery.com/graph/histogram_binSize.html

function renderData(data) {
    let dataSF = data.filter(el => el.city === "San Francisco");
    dataSF = dataSF.map(el => ({ dock_count: el.dock_count }));
    let dt = data.map(el => ({ dock_count: el.dock_count }));
    // console.log(dt);

    const width = 500;
    const height = 400;

    const margin = ({ top: 45, right: 45, bottom: 45, left: 45 });
    var svg = d3.select("#histogram")
        .append("svg")
        .attr("width", width)
        .attr("height", height)
        .append("g")
        .attr("transform", `translate(${margin.left},${margin.top})`);

    let sfDocks = dataSF.map(el => el.dock_count);
    let dtCount = dt.map(el => el.dock_count);

    const x = d3.scaleLinear()
        // .domain([0, d3.max(sfDocks)])  // - just for SF
        .domain([0, d3.max(dtCount)])
        .range([0, width - margin.left - margin.right])
        .nice();

    svg.append("g")
        .attr(
            "transform",
            `translate(0,${height - margin.top - margin.bottom})`)
        .call(d3.axisBottom(x));

    const y = d3.scaleLinear()
        .range([height - margin.top - margin.bottom, 0])
        .nice();

    const yAxis = svg.append("g");
    svg.append("text")
        .attr("fill", "black")
        .attr("x", 0)
        .attr("y", "-15")
        .style("font-size", "18px")
        .text("Distribution of bikes on dock stations in California. 2013 year.");

    svg.append("text")
        .attr("fill", "black")
        .attr("x", width / 2 - 50)
        .attr("y", height - 50)
        .style("font-size", "14px")
        .text("Bikes");

    svg.append("text")
        .attr("fill", "black")
        .attr("x", "120")
        .attr("y", -55)
        .attr("transform", "translate(30, 300) rotate(270)")
        .style("font-size", "14px")
        .text("Count");

    // function to act on the bin change
    function update(numBins) {

        // Set parameters for histogram, since v6 - bin()
        const hist = d3.bin()
            .value(d => d.dock_count)
            .domain(x.domain())
            .thresholds(x.ticks(numBins));

        // Get bins
        // const bins = hist(dataSF); - just for SF
        const bins = hist(dt); // for all dock stations
        y.domain([0, d3.max(bins, d => d.length)]);
        // console.log([0, d3.max(bins, d => d.length)]);

        yAxis.transition()
            .duration(200)
            .call(d3.axisLeft(y));

        const bars = svg.selectAll("rect")
            .data(bins);

        bars.join("rect")
            .transition()
            .duration(300)
            .attr("x", 1)
            .attr("transform", function(d) { return `translate(${x(d.x0)} , ${y(d.length)})` })
            .attr("width", function(d) { return x(d.x1) - x(d.x0) - 1; })
            .attr("height", function(d) { return height - margin.top - margin.bottom - y(d.length); })
            .style("fill", "#7cb310");
    }

    // Initial call
    update(5);

    // Waiting for the update to change bins
    d3.select("#numBins").on("input", function() {
        update(+this.value);
    });
}


d3.csv(
    "https://gist.githubusercontent.com/olga-kondr/64dc8df90513ada23e183cbb9fa1f9d4/raw/a69aa9c03948a74bb7453bdbcc7a3c70d81d9d9e/weather.csv",
).then(data => renderData(data));